#uses "fwUnitTestComponentAsserts.ctl"
#uses "test/fwRPC/utils.ctl"

#uses "fwRPC/fwRPC_pmon.ctl"


void test_fwRPC_pmon_setupSuite() {
  _BEGIN_SUITE("pmon");
}


void test_fwRPC_pmon_teardownSuite() {
  _END_SUITE();
}


void test_fwRPC_pmon_setup() {
  _BEGIN_TEST();
}

void test_fwRPC_pmon_teardown() {

}


//------------------------------------------------------------------------------


private global const string WORKER_SCRIPT = "fwRPC/fwRPC_worker_script.ctl";


//------------------------------------------------------------------------------


void test_fwRPC_pmon_retrieves_pmon_connection_info() {
  bool thrown = false;

  dyn_mixed coninfo;
  try {
    coninfo = fwRPC_pmon_getConInfo();
  } catch {
    thrown = true;
  }
  
  assertFalse(thrown, "should not throw");

  assertEqual(
    fwInstallation_getHostname(), coninfo[fwRPC_PMON_CONINFO_HOST],
    "should contain host info"
  );

  assertEqual(
    pmonPort(), coninfo[fwRPC_PMON_CONINFO_PORT],
    "should contain port info"
  );
}


void test_fwRPC_pmon_retrieves_manager_list() {
  dyn_mixed coninfo = fwRPC_pmon_getConInfo();

  dyn_dyn_mixed list;
  fwRPC_pmon_getManagerList(coninfo, list);

  assertTrue(dynlen(list) > 0, "should return a list");
}


void test_fwRPC_pmon_retrieves_manager_stati() {
  dyn_mixed coninfo = fwRPC_pmon_getConInfo();

  dyn_dyn_mixed list;
  fwRPC_pmon_getManagerStati(coninfo, list);

  assertTrue(dynlen(list) > 0, "should return a list");
}


void test_fwRPC_pmon_returns_manager_count() {
  dyn_mixed coninfo = fwRPC_pmon_getConInfo();

  int count = fwRPC_pmon_getManagerCount(coninfo);

  assertTrue(count > 0, "should return a manager count");
}


void test_fwRPC_pmon_appends_ctrl_manager() {
  dyn_mixed coninfo = fwRPC_pmon_getConInfo();

  int count = fwRPC_pmon_getManagerCount(coninfo);

  int index = fwRPC_pmon_appendCtrlManager(coninfo, WORKER_SCRIPT);

  assertEqual(count + 1, fwRPC_pmon_getManagerCount(coninfo));

  dyn_dyn_mixed managers;
  fwRPC_pmon_getManagerList(coninfo, managers);

  bool found = false;
  for(int i=1; i<=dynlen(managers); i++) {
    if(dynlen(managers[i]) < fwRPC_PMON_LIST_CMD) continue;

    if(managers[i][fwRPC_PMON_LIST_CMD] == WORKER_SCRIPT && i == index+1) {
      found = true;
      break;
    }
  }

  assertTrue(found, "Manager should be appended");

  //let's assume this works :)
  fwRPC_pmon_deleteCtrlManager(coninfo, index);
}


void test_fwRPC_pmon_starts_and_stops_ctrl_manager() {
  dyn_mixed coninfo = fwRPC_pmon_getConInfo();

  int index = fwRPC_pmon_appendCtrlManager(coninfo, WORKER_SCRIPT);

  fwRPC_pmon_startCtrlManager(coninfo, index);

  dyn_dyn_mixed managers;
  fwRPC_pmon_getManagerStati(coninfo, managers);
  assertEqual(
    fwRPC_PMON_MANINFO_STATE_RUNNING, managers[index+1][fwRPC_PMON_STAT_STATE]
  );

  fwRPC_pmon_stopCtrlManager(coninfo, index);

  fwRPC_pmon_getManagerStati(coninfo, managers);
  assertEqual(
    fwRPC_PMON_MANINFO_STATE_STOPPED, managers[index+1][fwRPC_PMON_STAT_STATE]
  );

  fwRPC_pmon_deleteCtrlManager(coninfo, index);
}


void test_fwRPC_pmon_deletes_ctrl_manager() {
  dyn_mixed coninfo = fwRPC_pmon_getConInfo();

  int index = fwRPC_pmon_appendCtrlManager(coninfo, WORKER_SCRIPT);

  fwRPC_pmon_deleteCtrlManager(coninfo, index);

  dyn_dyn_mixed managers;
  fwRPC_pmon_getManagerList(coninfo, managers);
  assertTrue(dynlen(managers) < index+1, "mngr should be deleted");
}


void test_fwRPC_pmon_retrieves_list_of_workers() {
  dyn_mixed coninfo = fwRPC_pmon_getConInfo();

  int index = fwRPC_pmon_appendCtrlManager(coninfo, WORKER_SCRIPT);

  dyn_dyn_mixed managers;
  fwRPC_pmon_getWorkerCtrlManagers(coninfo, managers);

  assertTrue(dynlen(managers) > 0, "Should return a list of managers");

  bool found = false;
  for(int i=1; i<=dynlen(managers); ++i) {
    if(managers[i][fwRPC_PMON_MANINFO_INDEX] == index) {
      found = true;
      assertEqual(WORKER_SCRIPT, managers[i][fwRPC_PMON_MANINFO_CMD]);
      break;
    }
  }

  assertTrue(found, "worker should be present in list");

  fwRPC_pmon_deleteCtrlManager(coninfo, index);
}


void test_fwRPC_pmon_deletes_the_last_worker() {
  dyn_mixed coninfo = fwRPC_pmon_getConInfo();

  int count = fwRPC_pmon_getManagerCount(coninfo);

  int index = fwRPC_pmon_appendCtrlManager(coninfo, WORKER_SCRIPT);

  assertEqual(count+1, fwRPC_pmon_getManagerCount(coninfo));

  fwRPC_pmon_deleteLastWorker(coninfo);

  assertEqual(count, fwRPC_pmon_getManagerCount(coninfo));
}
